import pytest
from orchestrator.types import SubscriptionLifecycle

from gso.products import ProductName
from gso.products.product_blocks.switch import SwitchModel
from gso.products.product_types.switch import ImportedSwitch
from gso.utils.shared_enums import Vendor
from test.workflows import (
    assert_complete,
    extract_state,
    run_workflow,
)


@pytest.fixture()
def workflow_input_data(faker, site_subscription_factory):
    return {
        "fqdn": faker.domain_name(levels=4),
        "ts_port": faker.port_number(is_user=True),
        "site": site_subscription_factory(),
        "switch_vendor": Vendor.JUNIPER,
        "switch_model": SwitchModel.EX3400,
    }


@pytest.mark.workflow()
def test_create_imported_switch_success(workflow_input_data):
    result, _, _ = run_workflow("create_imported_switch", [workflow_input_data])
    state = extract_state(result)
    subscription = ImportedSwitch.from_subscription(state["subscription_id"])

    assert_complete(result)
    assert subscription.product.name == ProductName.IMPORTED_SWITCH
    assert subscription.status == SubscriptionLifecycle.ACTIVE
